==> ns-1.0b4.tar.gz: ns-1.0b4/loss-monitor.cc <==
/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: loss-monitor.cc,v 1.7 96/02/19 14:33:15 mccanne Exp $ (LBL)";
#endif

#include "agent.h"
#include "node.h"
#include "Tcl.h"
#include "packet.h"
#include "trace.h"

class LossMonitor : public Agent {
 public:
	LossMonitor();
	int command(int argc, const char*const* argv);
	void recv(Packet* pkt);
protected:
	Trace* trace_;
	char* proc_;
	int nlost_;
	int npkts_;
	int expected_;
	int bytes_;
	double last_packet_time_;
};

static class LossMonitorMatcher : public Matcher {
public:
	LossMonitorMatcher() : Matcher("agent") {}
	TclObject* match(const char* id) {
		if (strcmp(id, "loss-monitor") == 0)
			return (new LossMonitor());
		return (0);
	}
} matcher_cbr;

LossMonitor::LossMonitor() : Agent(-1), proc_(0), expected_(-1), trace_(0)
{
	bytes_ = 0;
	nlost_ = 0;
	npkts_ = 0;
	last_packet_time_ = 0.;
	link_int(0, "nlost", &nlost_, 1);
	link_int(0, "npkts", &npkts_, 1);
	link_int(0, "bytes", &bytes_, 1);
	link_real(0, "last-packet-time", &last_packet_time_, 1);
}

void LossMonitor::recv(Packet* pkt)
{
	bytes_ += pkt->size_;
	++npkts_;
	/*
	 * Check for lost packets
	 */
	last_packet_time_ = Scheduler::instance().clock();
	if (expected_ >= 0) {
		int loss = pkt->seqno_ - expected_;
		if (loss > 0) {
			nlost_ += loss;
			if (proc_ != 0)
				Tcl::instance().eval(proc_);

			if (trace_ != 0) {
				sprintf(trace_->buffer(), "d %g %d",
					last_packet_time_, loss);
				trace_->dump();
			}
		}
	}
	if (trace_ != 0) {
		sprintf(trace_->buffer(), "r %g", last_packet_time_);
		trace_->dump();
	}
	expected_ = pkt->seqno_ + 1;
	Packet::free(pkt);
}

/*
 * $proc interval $interval
 * $proc size $size
 */
int LossMonitor::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "clear") == 0) {
			expected_ = -1;
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "proc") == 0) {
			const char* proc = argv[2];
			int n = strlen(proc) + 1;
			delete proc_;
			proc_ = new char[n];
			strcpy(proc_, proc);
			return (TCL_OK);
		}
		if (strcmp(argv[1], "trace") == 0) {
			trace_ = (Trace*)TclObject::lookup(argv[2]);
			if (trace_ == 0) {
				Tcl& tcl = Tcl::instance();
				tcl.resultf("no trace %s", argv[2]);
				return (TCL_ERROR);
			}
			return (TCL_OK);
		}
	}
	return (Agent::command(argc, argv));
}

==> ns-1.1.tar.gz: ns-1.1/loss-monitor.cc <==
/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /n/olympus/opt/users/CVS/ns-1.1-master/loss-monitor.cc,v 1.2 1997/09/08 20:51:00 wuchang Exp $ (LBL)";
#endif

#include "agent.h"
#include "node.h"
#include "Tcl.h"
#include "packet.h"
#include "trace.h"

class LossMonitor : public Agent {
 public:
	LossMonitor();
	int command(int argc, const char*const* argv);
	void recv(Packet* pkt);
protected:
	char* proc_;
	int nlost_;
	int npkts_;
	int expected_;
	int bytes_;
	double last_packet_time_;
};

static class LossMonitorMatcher : public Matcher {
public:
	LossMonitorMatcher() : Matcher("agent") {}
	TclObject* match(const char* id) {
		if (strcmp(id, "loss-monitor") == 0)
			return (new LossMonitor());
		return (0);
	}
} matcher_cbr;

LossMonitor::LossMonitor() : Agent(-1), proc_(0), expected_(-1)
{
	bytes_ = 0;
	nlost_ = 0;
	npkts_ = 0;
	last_packet_time_ = 0.;
	link_int(0, "nlost", &nlost_, 1);
	link_int(0, "npkts", &npkts_, 1);
	link_int(0, "bytes", &bytes_, 1);
	link_real(0, "last-packet-time", &last_packet_time_, 1);
}

void LossMonitor::recv(Packet* pkt)
{
	double now = Scheduler::instance().clock();
	bytes_ += pkt->size_;
	++npkts_;
	/*
	 * Check for lost packets
	 */
	if (expected_ >= 0) {
		int loss = pkt->seqno_ - expected_;
		if (loss > 0) {
			nlost_ += loss;
			if (proc_ != 0)
				Tcl::instance().eval(proc_);

			if (trace_ != 0) {
				sprintf(trace_->buffer(), "d %g %d %d",
					now, expected_, loss);
				trace_->dump();
			}
		}
	}
#ifdef notdef
	if (trace_ != 0) {
		sprintf(trace_->buffer(), "r %g", last_packet_time_);
		trace_->dump();
	}
#endif
	expected_ = pkt->seqno_ + 1;
	last_packet_time_ = now;
	Packet::free(pkt);
}

/*
 * $proc interval $interval
 * $proc size $size
 */
int LossMonitor::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "clear") == 0) {
			expected_ = -1;
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "proc") == 0) {
			const char* proc = argv[2];
			int n = strlen(proc) + 1;
			delete proc_;
			proc_ = new char[n];
			strcpy(proc_, proc);
			return (TCL_OK);
		}
	}
	return (Agent::command(argc, argv));
}
